<?php
namespace InternetGalerie\Igshop2\Widgets;

use InternetGalerie\Igshop2\Widgets\Provider\ProductProviderInterface;
use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
use TYPO3\CMS\Dashboard\Widgets\AdditionalCssInterface;
use TYPO3\CMS\Dashboard\Widgets\ButtonProviderInterface;
use TYPO3\CMS\Dashboard\Widgets\ListDataProviderInterface;
use TYPO3\CMS\Dashboard\Widgets\WidgetConfigurationInterface;
use TYPO3\CMS\Dashboard\Widgets\WidgetInterface;
use TYPO3\CMS\Fluid\View\StandaloneView;

class ProductWidget implements WidgetInterface
{
    /**
     * @var WidgetConfigurationInterface
     */
    private $configuration;

    /**
     * @var StandaloneView
     */
    private $view;

    /**
     * @var ProductProviderInterface
     */
    private $dataProvider;

    /**
     * @var array
     */
    private $options;

    /**
     * @var ButtonProviderInterface|null
     */
    private $buttonProvider;

    public function __construct(
        WidgetConfigurationInterface $configuration,
        ProductProviderInterface $dataProvider,
        StandaloneView $view,
        ButtonProviderInterface $buttonProvider = null,
        array $options = []
    )
    {
        $this->configuration = $configuration;
        $this->view = $view;
        $this->dataProvider = $dataProvider;
        $this->options =  array_merge(
            [
                'template' => 'IgShop2/Widgets/ProductWidget',
            ],
            $options
        );
        $this->buttonProvider = $buttonProvider;
    }

    public function renderWidgetContent(): string
    {
        $this->view->setTemplate($this->options['template']);
        $this->view->assignMultiple([
            'categoriesLimit' => $this->dataProvider->getLimit(),
            'categories' => $this->dataProvider->getCategories(  ),
            'total' => $this->dataProvider->getProductTotal(),
            'options' => $this->options,
            //'button' => $this->getButton(),
            'configuration' => $this->configuration,
        ]);
        return $this->view->render();
    }

    public function getOptions(): array
    {
        return $this->options;
    }
}
